@@ -1,5 +1,5 @@
 /*
- * CU sudo version 1.4 (based on Root Group sudo version 1.1)
+ * CU sudo version 1.5 (based on Root Group sudo version 1.1)
  *
  * This software comes with no waranty whatsoever, use at your own risk.
  *
@@ -36,7 +36,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: check.c,v 1.97 1996/01/14 20:19:22 millert Exp $";
+static char rcsid[] = "$Id: check.c,v 1.110 1996/08/17 23:25:40 millert Exp $";
 #endif /* lint */
 
 #include "config.h"
@@ -66,16 +66,19 @@ static char rcsid[] = "$Id: check.c,v 1.97 1996/01/14 20:19:22 millert Exp $";
 #include "sudo.h"
 #include <options.h>
 #include "insults.h"
-#if defined(SHADOW_TYPE) && (SHADOW_TYPE == SPW_SECUREWARE)
-#  include <sys/security.h>
+#if (SHADOW_TYPE == SPW_SECUREWARE)
+#  ifdef __hpux
+#    include <hpsecurity.h>
+#  else
+#    include <sys/security.h>
+#  endif /* __hpux */
 #  include <prot.h>
-#endif /* SHADOW_TYPE == SPW_SECUREWARE */
+#endif /* SPW_SECUREWARE */
 #ifdef HAVE_KERB4
 #  include <krb.h>
 #endif /* HAVE_KERB4 */
 #ifdef HAVE_AFS
-#  include <usersec.h>
-#  include <afs/kauth.h>
+#  include <afs/stds.h>
 #  include <afs/kautils.h>
 #endif /* HAVE_AFS */
 #ifdef HAVE_SECURID
@@ -343,7 +346,8 @@ static void update_timestamp()
 
 	    if (fd < 0)
 		perror("update_timestamp: open");
-	    close(fd);
+	    else
+		close(fd);
 	}
 
 	/* relinquish root */
@@ -401,7 +405,7 @@ static void check_passwd()
     struct SD_CLIENT sd_dat, *sd;		/* SecurID data block */
     register int counter = TRIES_FOR_PASSWORD;
 
-    (void) memset (&sd_dat, 0, sizeof(sd_dat));
+    (void) memset ((VOID *)&sd_dat, 0, sizeof(sd_dat));
     sd = &sd_dat;
 
     /* Initialize SecurID. */
@@ -445,32 +449,27 @@ static void check_passwd()
 {
     char *pass;			/* this is what gets entered    */
     register int counter = TRIES_FOR_PASSWORD;
-#ifdef HAVE_AFS
-    int code;
-    long password_expires = -1;
-    char *reason;
-#endif /* HAVE_AFS */
-#if defined(SHADOW_TYPE) && (SHADOW_TYPE == SPW_SECUREWARE)
-    char salt[2];		/* Need the salt to perform the encryption */
-    register int i;
-#endif /* SHADOW_TYPE == SECUREWARE */
 #if defined(HAVE_KERB4) && defined(USE_GETPASS)
-    char kpass[_PASSWD_LEN];
+    char kpass[_PASSWD_LEN + 1];
 #endif /* HAVE_KERB4 && USE_GETPASS */
 
+#ifdef HAVE_SKEY
+    (void) memset((VOID *)&skey, 0, sizeof(skey));
+#endif /* HAVE_SKEY */
+
     /*
      * you get TRIES_FOR_PASSWORD times to guess your password
      */
     while (counter > 0) {
 
 #ifdef HAVE_SKEY
-    /* rewrite the prompt if using s/key since the challenge can change */
-    set_perms(PERM_ROOT);
-    prompt = sudo_skeyprompt(&skey, prompt);
-    set_perms(PERM_USER);
+	/* rewrite the prompt if using s/key since the challenge can change */
+	set_perms(PERM_ROOT);
+	prompt = sudo_skeyprompt(&skey, prompt);
+	set_perms(PERM_USER);
 #endif /* HAVE_SKEY */
 
-    /* get a password from the user */
+	/* get a password from the user */
 #ifdef USE_GETPASS
 #  ifdef HAVE_KERB4
 	(void) des_read_pw_string(kpass, sizeof(kpass) - 1, prompt, 0);
@@ -492,7 +491,7 @@ static void check_passwd()
 
 #ifdef HAVE_SKEY
 	/* Only check s/key db if the user exists there */
-	if (skey.logname) {
+	if (skey.keyfile) {
 	    set_perms(PERM_ROOT);
 	    if (skeyverify(&skey, pass) == 0) {
 		set_perms(PERM_USER);
@@ -506,16 +505,19 @@ static void check_passwd()
 	 * If we use shadow passwords with a different crypt(3)
 	 * check that here, else use standard crypt(3).
 	 */
-#  ifdef SHADOW_TYPE
+#  if (SHADOW_TYPE != SPW_NONE) && (SHADOW_TYPE != SPW_BSD)
 #    if (SHADOW_TYPE == SPW_ULTRIX4)
-	if (!strcmp(user_passwd, (char *)crypt16(pass, user_passwd)))
+	if (!strcmp(user_passwd, (char *) crypt16(pass, user_passwd)))
 	    return;		/* if the passwd is correct return() */
 #    endif /* ULTRIX4 */
 #    if (SHADOW_TYPE == SPW_SECUREWARE) && !defined(__alpha)
-	strncpy(salt, user_passwd, 2);
-	i = AUTH_SALT_SIZE + AUTH_CIPHERTEXT_SEG_CHARS;
-	if (strncmp(user_passwd, crypt(pass, salt), i) == 0)
+#      ifdef HAVE_BIGCRYPT
+	if (strcmp(user_passwd, (char *) bigcrypt(pass, user_passwd)) == 0)
 	    return;           /* if the passwd is correct return() */
+#      else
+	if (strcmp(user_passwd, crypt(pass, user_passwd)) == 0)
+	    return;           /* if the passwd is correct return() */
+#      endif /* HAVE_BIGCRYPT */
 #    endif /* SECUREWARE && !__alpha */
 #    if (SHADOW_TYPE == SPW_SECUREWARE) && defined(__alpha)
 	if (crypt_type == AUTH_CRYPT_BIGCRYPT) {
@@ -524,14 +526,20 @@ static void check_passwd()
 	} else if (crypt_type == AUTH_CRYPT_CRYPT16) {
 	    if (!strcmp(user_passwd, crypt16(pass, user_passwd)))
 		return;             /* if the passwd is correct return() */
+#ifdef AUTH_CRYPT_OLDCRYPT
+	} else if (crypt_type == AUTH_CRYPT_OLDCRYPT ||
+		   crypt_type == AUTH_CRYPT_C1CRYPT) {
+	    if (!strcmp(user_passwd, crypt(pass, user_passwd)))
+		return;             /* if the passwd is correct return() */
+#endif
 	} else {
 	    (void) fprintf(stderr,
                     "%s: Sorry, I don't know how to deal with crypt type %d.\n",
-                    Argv[0]);
+                    Argv[0], crypt_type);
 	    exit(1);
 	}
 #    endif /* SECUREWARE && __alpha */
-#  endif /* SHADOW_TYPE */
+#  endif /* SHADOW_TYPE != SPW_NONE && SHADOW_TYPE != SPW_BSD */
 
 	/* Normal UN*X password check */
 	if (!strcmp(user_passwd, (char *) crypt(pass, user_passwd)))
@@ -543,16 +551,14 @@ static void check_passwd()
 #  endif /* HAVE_KERB4 */
 
 #  ifdef HAVE_AFS
-	code = ka_UserAuthenticateGeneral(KA_USERAUTH_VERSION+KA_USERAUTH_DOSETPAG,
-                                          user_name,
-                                          (char *) 0, 
-                                          (char *) 0,
-                                          pass,
-                                          0,
-                                          &password_expires,
-                                          0,
-                                          &reason);
-	if (code == 0)
+	if (ka_UserAuthenticateGeneral(KA_USERAUTH_VERSION,
+                                       user_name,	/* name */
+                                       NULL,		/* instance */
+                                       NULL,		/* realm */
+                                       pass,		/* password */
+                                       0,		/* lifetime */
+                                       0, 0,		/* spare */
+                                       NULL) == 0)	/* reason */
 	    return;
 #  endif /* HAVE_AFS */
 #  ifdef HAVE_DCE
@@ -652,33 +658,45 @@ static char *sudo_skeyprompt(user_skey, p)
     struct skey *user_skey;
     char *p;
 {
-    char skeyprompt[80];
 #ifndef LONG_SKEY_PROMPT
+    char buf[32];
+    char *new_prompt;
     static char *old_prompt = NULL;
     static int plen;
-    char *new_prompt;
 #endif /* LONG_SKEY_PROMPT */
 
+    /* close the key file if necesary */
+    if (user_skey->keyfile != NULL)
+	(void) fclose(user_skey->keyfile);
+
     /* return the old prompt if we cannot get s/key info */
-    if (skeychallenge(user_skey, user_name, skeyprompt)) {
-#  ifdef SKEY_ONLY
+    if (skeylookup(user_skey, user_name)) {
+	if (user_skey->keyfile != NULL) {
+	    (void) fclose(user_skey->keyfile);
+	    user_skey->keyfile = NULL;
+	}
+#ifdef SKEY_ONLY
 	(void) fprintf(stderr, "%s: You do not exist in the s/key database.\n",
 		       Argv[0]);
 	exit(1);
-#  else
-	user_skey->logname = NULL;
+#else
+#  ifdef LONG_SKEY_PROMPT
 	return(p);
-#  endif /* SKEY_ONLY */
+#  else
+	if (old_prompt == NULL) {
+	    return(p);
+	} else {
+	    return(old_prompt);
+	}
+#  endif /* LONG_SKEY_PROMPT */
+#endif /* SKEY_ONLY */
     }
 
 #ifdef LONG_SKEY_PROMPT
     /* separate s/key challenge and prompt for easy snarfing */
-    if (skeyprompt[0] == 's' && skeyprompt[1] == '/')
-	(void) puts(&skeyprompt[2]);
-    else
-	(void) puts(skeyprompt);
+    (void) printf("key %d %s\n", user_skey->n - 1, user_skey->seed);
 
-    /* return old prompt unmodified */
+    /* return old prompt unmolested */
     return(p);
 
 #else
@@ -695,15 +713,17 @@ static char *sudo_skeyprompt(user_skey, p)
 	(void) free(p);
     }
 
-    if ((new_prompt = (char *) malloc(plen + strlen(skeyprompt) + 5)) == NULL) {
+    (void) sprintf(buf, "%d", user_skey->n - 1);
+    if ((new_prompt = (char *)
+	malloc(plen + strlen(buf) + strlen(user_skey->seed) + 12)) == NULL) {
 	perror("malloc");
 	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	exit(1);
     }
 
     /* embed the s/key challenge into the new password prompt */
-    (void) strncpy(new_prompt, old_prompt, plen);
-    (void) sprintf(new_prompt + plen, " [%s]:", skeyprompt);
+    (void) sprintf(new_prompt, "%.*s [s/key %d %s]:", plen, old_prompt,
+	user_skey->n - 1, user_skey->seed);
 
     return(new_prompt);
 #endif /* LONG_SKEY_PROMPT */
@@ -724,7 +744,7 @@ static void reminder()
     (void) fprintf(stderr, "\n%s\n%s\n\n%s\n%s\n\n",
 #else
     (void) fprintf(stderr, "\n%s\n%s\n%s\n%s\n\n%s\n%s\n\n%s\n%s\n\n",
-	"    CU sudo version 1.4, based on Root Group sudo version 1.1",
+	"    CU sudo version 1.5, based on Root Group sudo version 1.1",
 	"    sudo version 1.1, Copyright (C) 1991 The Root Group, Inc.",
 	"    sudo comes with ABSOLUTELY NO WARRANTY.  This is free software,",
 	"    and you are welcome to redistribute it under certain conditions.",
