@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.4
+ *  CU sudo version 1.5
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: find_path.c,v 1.56 1996/01/14 20:19:22 millert Exp $";
+static char rcsid[] = "$Id: find_path.c,v 1.60 1996/08/17 23:25:40 millert Exp $";
 #endif /* lint */
 
 #include "config.h"
@@ -99,7 +99,9 @@ char * find_path(file)
     char *path = NULL;		/* contents of PATH env var */
     char *origpath;		/* so we can free path later */
     char *result = NULL;	/* result of path/file lookup */
+#ifndef IGNORE_DOT_PATH
     int checkdot = 0;		/* check current dir? */
+#endif /* IGNORE_DOT_PATH */
 
     command[0] = '\0';
 
@@ -148,7 +150,9 @@ char * find_path(file)
 	 * things like using './' or './/' 
 	 */
 	if (*path == '\0' || (*path == '.' && *(path + 1) == '\0')) {
+#ifndef IGNORE_DOT_PATH
 	    checkdot = 1;
+#endif /* IGNORE_DOT_PATH */
 	    path = n + 1;
 	    continue;
 	}
@@ -169,13 +173,13 @@ char * find_path(file)
 
     } while (n);
 
+#ifndef IGNORE_DOT_PATH
     /*
      * check current dir if dot was in the PATH
      */
-    if (!result && checkdot) {
-	path = ".";
-	result = sudo_goodpath(command);
-    }
+    if (!result && checkdot)
+	result = sudo_goodpath(file);
+#endif /* IGNORE_DOT_PATH */
 
     (void) free(origpath);
 
