@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.4
+ *  CU sudo version 1.5
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: interfaces.c,v 1.19 1996/01/14 20:16:46 millert Exp $";
+static char rcsid[] = "$Id: interfaces.c,v 1.23 1996/08/17 23:25:40 millert Exp $";
 #endif /* lint */
 
 #include "config.h"
@@ -163,8 +163,8 @@ void load_interfaces()
 
 	/* set i to the subscript of the next interface */
 #ifdef HAVE_SA_LEN
-	if (ifreq.ifr_addr.sa_len > sizeof(ifreq.ifr_addr))
-	    i += sizeof(ifreq.ifr_name) + ifreq.ifr_addr.sa_len;
+	if (ifr->ifr_addr.sa_len > sizeof(ifr->ifr_addr))
+	    i += sizeof(ifr->ifr_name) + ifr->ifr_addr.sa_len;
 	else
 #endif /* HAVE_SA_LEN */
 	    i += sizeof(struct ifreq);
