@@ -1,5 +1,5 @@
 /*
- * CU sudo version 1.4 (based on Root Group sudo version 1.1)
+ * CU sudo version 1.5 (based on Root Group sudo version 1.1)
  *
  * This software comes with no waranty whatsoever, use at your own risk.
  *
@@ -37,7 +37,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: logging.c,v 1.69 1996/01/14 20:19:56 millert Exp $";
+static char rcsid[] = "$Id: logging.c,v 1.80 1996/08/17 23:25:40 millert Exp $";
 #endif /* lint */
 
 #include "config.h"
@@ -87,6 +87,11 @@ static void syslog_wrapper	__P((int, char *, char *, char *));
 static char *logline;
 extern int errorlineno;
 
+/*
+ * length of syslog-like header info used for mail and file logs
+ * is len("Mon MM HH:MM:SS : username : ")
+ */
+#define LOG_HEADER_LEN		29
 
 #ifdef BROKEN_SYSLOG
 #define MAXSYSLOGTRIES		16	/* num of retries for broken syslogs */
@@ -144,9 +149,9 @@ void log_error(code)
 
     /*
      * Allocate enough memory for logline so we won't overflow it
-     * XXX - don't use 33, use a macro!
      */
-    count = 33 + 128 + 2 * MAXPATHLEN + strlen(tty) + strlen(cwd);
+    count = LOG_HEADER_LEN + 136 + 2 * MAXPATHLEN + strlen(tty) + strlen(cwd) +
+	    strlen(runas_user);
     if (cmnd_args)
 	count += strlen(cmnd_args);
 
@@ -162,17 +167,19 @@ void log_error(code)
      * necesary for mail and file logs.
      */
     now = time((time_t) 0);
-    (void) sprintf(logline, "%19.19s : %8.8s : ", ctime(&now), user_name);
+    p = ctime(&now) + 4;
+    (void) sprintf(logline, "%15.15s : %8.8s : ", p, user_name);
 
     /*
-     * we need a pointer to the end of logline (XXX - use a #define not 33)
+     * we need a pointer to the end of logline for cheap appends.
      */
-    p = logline + 33;
+    p = logline + LOG_HEADER_LEN;
 
     switch (code) {
 
 	case ALL_SYSTEMS_GO:
-	    (void) sprintf(p, "TTY=%s ; PWD=%s ; COMMAND=", tty, cwd);
+	    (void) sprintf(p, "TTY=%s ; PWD=%s ; USER=%s ; COMMAND=",
+	    tty, cwd, runas_user);
 #if (LOGGING & SLOG_SYSLOG)
 	    pri = Syslog_priority_OK;
 #endif /* LOGGING & SLOG_SYSLOG */
@@ -180,17 +187,19 @@ void log_error(code)
 
 	case VALIDATE_NO_USER:
 	    (void) sprintf(p,
-		"user NOT in sudoers ; TTY=%s ; PWD=%s ; COMMAND=", tty, cwd);
+		"user NOT in sudoers ; TTY=%s ; PWD=%s ; USER=%s ; COMMAND=",
+		tty, cwd, runas_user);
 	    break;
 
 	case VALIDATE_NOT_OK:
 	    (void) sprintf(p,
-		"command not allowed ; TTY=%s ; PWD=%s ; COMMAND=", tty, cwd);
+		"command not allowed ; TTY=%s ; PWD=%s ; USER=%s ; COMMAND=",
+		tty, cwd, runas_user);
 	    break;
 
 	case VALIDATE_ERROR:
-	    (void) sprintf(p, "error in %s, line %d ; TTY=%s ; PWD=%s.  ",
-		_PATH_SUDO_SUDOERS, errorlineno, tty, cwd);
+	    (void) sprintf(p, "error in %s, line %d ; TTY=%s ; PWD=%s ; USER=%s.  ",
+		_PATH_SUDO_SUDOERS, errorlineno, tty, cwd, runas_user);
 	    break;
 
 	case GLOBAL_NO_PW_ENT:
@@ -201,13 +210,14 @@ void log_error(code)
 
 	case PASSWORD_NOT_CORRECT:
 	    (void) sprintf(p,
-		"password incorrect ; TTY=%s ; PWD=%s ; COMMAND=", tty, cwd);
+		"password incorrect ; TTY=%s ; PWD=%s ; USER=%s ; COMMAND=",
+		tty, cwd, runas_user);
 	    break;
 
 	case PASSWORDS_NOT_CORRECT:
 	    (void) sprintf(p,
-		"%d incorrect passwords ; TTY=%s ; PWD=%s ; COMMAND=",
-		    TRIES_FOR_PASSWORD, tty, cwd);
+		"%d incorrect passwords ; TTY=%s ; PWD=%s ; USER=%s ; COMMAND=",
+		TRIES_FOR_PASSWORD, tty, cwd, runas_user);
 	    break;
 
 	case GLOBAL_NO_HOSTNAME:
@@ -221,7 +231,7 @@ void log_error(code)
 			_PATH_SUDO_SUDOERS);
 		    break;
 		case EACCES:
-		    (void) sprintf(p, "Can't read %s.  ", Argv[0]);
+		    (void) sprintf(p, "Can't read %s.  ", _PATH_SUDO_SUDOERS);
 		    break;
 		default:
 		    (void) sprintf(p, "There is a problem opening %s ",
@@ -234,41 +244,36 @@ void log_error(code)
 	    (void) sprintf(p, "gethostbyname() cannot find host %s ", host);
 	    break;
 
-	case SUDOERS_NO_OWNER:
-	    (void) sprintf(p, "no passwd entry for sudoers file owner (%s) ",
-		SUDOERS_OWNER);
-	    break;
-
 	case SUDOERS_NOT_FILE:
 	    (void) sprintf(p, "%s is not a regular file ", _PATH_SUDO_SUDOERS);
 	    break;
 
 	case SUDOERS_WRONG_OWNER:
-	    (void) sprintf(p, "%s is not owned by %s ", _PATH_SUDO_SUDOERS,
-		SUDOERS_OWNER);
+	    (void) sprintf(p, "%s is not owned by uid %d and gid %d ",
+		_PATH_SUDO_SUDOERS, SUDOERS_UID, SUDOERS_GID);
 	    break;
 
-	case SUDOERS_RW_OTHER:
-	    (void) sprintf(p, "%s is readable or writeable by other than %s ",
-		_PATH_SUDO_SUDOERS, SUDOERS_OWNER);
+	case SUDOERS_WRONG_MODE:
+	    (void) sprintf(p, "%s is not mode %o ", _PATH_SUDO_SUDOERS,
+		SUDOERS_MODE);
 	    break;
 
 	case SPOOF_ATTEMPT:
 	    (void) sprintf(p,
-		"probable spoofing attempt; TTY=%s ; PWD=%s ; COMMAND=",
-		tty, cwd);
+		"probable spoofing attempt; TTY=%s ; PWD=%s ; USER=%s ; COMMAND=",
+		tty, cwd, runas_user);
 	    break;
 
 	case BAD_STAMPDIR:
 	    (void) sprintf(p,
-	    "%s owned by non-root or not mode 0700; TTY=%s ; PWD=%s ; COMMAND=",
-	    _PATH_SUDO_TIMEDIR, tty, cwd);
+	    "%s owned by non-root or not mode 0700; TTY=%s ; PWD=%s ; USER=%s ; COMMAND=",
+	    _PATH_SUDO_TIMEDIR, tty, cwd, runas_user);
 	    break;
 
 	case BAD_STAMPFILE:
 	    (void) sprintf(p,
-		"preposterous stampfile date; TTY=%s ; PWD=%s ; COMMAND=",
-		tty, cwd);
+		"preposterous stampfile date; TTY=%s ; PWD=%s ; USER=%s ; COMMAND=",
+		tty, cwd, runas_user);
 	    break;
 
 	default:
@@ -284,12 +289,20 @@ void log_error(code)
     if (code != VALIDATE_ERROR && !(code & GLOBAL_PROBLEM)) {
 
 	/* stuff the command into the logline */
-	/* XXX - this could be sped up */
-	strcat(logline, cmnd);
-	strcat(logline, " ");
+	p = logline + strlen(logline);
+	strcpy(p, cmnd);
+
+	/* add a trailing space */
+	p += strlen(cmnd);
+	*p++ = ' ';
+	*p = '\0';
+
+	/* cat on command args if they exist */
 	if (cmnd_args) {
-	    strcat(logline, cmnd_args);
-	    strcat(logline, " ");
+	    (void) strcpy(p, cmnd_args);
+	    p += strlen(cmnd_args);
+	    *p++ = ' ';
+	    *p = '\0';
 	}
     }
 
@@ -303,13 +316,12 @@ void log_error(code)
     /*
      * Log the full line, breaking into multiple syslog(3) calls if necesary
      */
-    p = &logline[33];			/* skip past the date and user */
+    p = &logline[LOG_HEADER_LEN];	/* skip past the date and user */
     for (count = 0; count < strlen(logline) / MAXSYSLOGLEN + 1; count++) {
 	if (strlen(p) > MAXSYSLOGLEN) {
 	    /*
 	     * Break up the line into what will fit on one syslog(3) line
 	     * Try to break on a word boundary if possible.
-	     * XXX - speed this up!
 	     */
 	    for (tmp = p + MAXSYSLOGLEN; tmp > p && *tmp != ' '; tmp--)
 		;
@@ -425,10 +437,7 @@ void log_error(code)
  *
  */
 
-static char *mail_argv[] = { "sendmail",
-			     "-t",
-			     ALERTMAIL,
-			     (char *) NULL };
+static char *mail_argv[] = { "sendmail", "-t", (char *) NULL };
 
 static void send_mail()
 {
@@ -535,7 +544,6 @@ static RETSIGTYPE reapchild(sig)
 void inform_user(code)
     int code;
 {
-
     switch (code) {
 	case VALIDATE_NO_USER:
 	    (void) fprintf(stderr,
@@ -544,14 +552,17 @@ void inform_user(code)
 	    break;
 
 	case VALIDATE_NOT_OK:
-	    if (cmnd_args)
-		(void) fprintf(stderr,
-		    "Sorry, user %s is not allowed to execute \"%s %s\" on %s.\n\n",
-		    user_name, cmnd, cmnd_args, host);
-	    else
-		(void) fprintf(stderr,
-		    "Sorry, user %s is not allowed to execute \"%s\" on %s.\n\n",
-		    user_name, cmnd, host);
+	    (void) fprintf(stderr,
+		"Sorry, user %s is not allowed to execute \"%s",
+		user_name, cmnd);
+	
+	    /* print command args if they exist */
+	    if (cmnd_args) {
+		fputc(' ', stderr);
+		fputs(cmnd_args, stderr);
+	    }
+
+	    (void) fprintf(stderr, "\" as %s on %s.\n\n", runas_user, host);
 	    break;
 
 	case VALIDATE_ERROR:
@@ -588,11 +599,6 @@ void inform_user(code)
 		TRIES_FOR_PASSWORD);
 	    break;
 
-	case SUDOERS_NO_OWNER:
-	    (void) fprintf(stderr,
-		"No passwd entry for sudoers file owner (%s)\n", SUDOERS_OWNER);
-	    break;
-
 	case NO_SUDOERS_FILE:
 	    switch (errno) {
 		case ENOENT:
@@ -613,14 +619,13 @@ void inform_user(code)
 	    break;
 
 	case SUDOERS_WRONG_OWNER:
-	    (void) fprintf(stderr, "%s is not owned by %s!\n",
-		_PATH_SUDO_SUDOERS, SUDOERS_OWNER);
+	    (void) fprintf(stderr, "%s is not owned by uid %d and gid %d!\n",
+		_PATH_SUDO_SUDOERS, SUDOERS_UID, SUDOERS_GID);
 	    break;
 
-	case SUDOERS_RW_OTHER:
-	    (void) fprintf(stderr,
-		"%s is readable or writeable by other than %s!\n",
-		_PATH_SUDO_SUDOERS, SUDOERS_OWNER);
+	case SUDOERS_WRONG_MODE:
+	    (void) fprintf(stderr, "%s must be mode %o!\n", _PATH_SUDO_SUDOERS,
+		SUDOERS_MODE);
 	    break;
 
 	case SPOOF_ATTEMPT:
@@ -665,6 +670,7 @@ static int appropriate(code)
      * these will NOT send mail
      */
     case VALIDATE_OK:
+    case VALIDATE_OK_NOPASS:
     case PASSWORD_NOT_CORRECT:
     case PASSWORDS_NOT_CORRECT:
 /*  case ALL_SYSTEMS_GO:               this is the same as OK */
