@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.4
+ *  CU sudo version 1.5
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,28 +25,39 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: parse.c,v 1.50 1996/01/14 20:15:20 millert Exp $";
+static char rcsid[] = "$Id: parse.c,v 1.72 1996/08/17 23:25:40 millert Exp $";
 #endif /* lint */
 
 #include "config.h"
 
 #include <stdio.h>
 #ifdef STDC_HEADERS
-#include <stdlib.h>
+#  include <stdlib.h>
 #endif /* STDC_HEADERS */
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
+#  include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #endif /* HAVE_STRING_H */
 #ifdef HAVE_STRINGS_H
-#include <strings.h>
+#  include <strings.h>
 #endif /* HAVE_STRINGS_H */
+#ifdef HAVE_FNMATCH_H
+#  include <fnmatch.h>
+#else
+#  ifndef HAVE_FNMATCH
+#    include "emul/fnmatch.h"
+#  endif /* HAVE_FNMATCH */
+#endif /* HAVE_FNMATCH_H */
 #if defined(HAVE_MALLOC_H) && !defined(STDC_HEADERS)
-#include <malloc.h>
+#  include <malloc.h>
 #endif /* HAVE_MALLOC_H && !STDC_HEADERS */
+#ifdef HAVE_NETGROUP_H
+#  include <netgroup.h>
+#endif /* HAVE_NETGROUP_H */
 #include <ctype.h>
+#include <pwd.h>
 #include <grp.h>
 #include <sys/param.h>
 #include <sys/types.h>
@@ -55,34 +66,40 @@ static char rcsid[] = "$Id: parse.c,v 1.50 1996/01/14 20:15:20 millert Exp $";
 #include <netdb.h>
 #include <sys/stat.h>
 #if HAVE_DIRENT_H
-# include <dirent.h>
-# define NAMLEN(dirent) strlen((dirent)->d_name)
+#  include <dirent.h>
+#  define NAMLEN(dirent) strlen((dirent)->d_name)
 #else
-# define dirent direct
-# define NAMLEN(dirent) (dirent)->d_namlen
-# if HAVE_SYS_NDIR_H
-#  include <sys/ndir.h>
-# endif
-# if HAVE_SYS_DIR_H
-#  include <sys/dir.h>
-# endif
-# if HAVE_NDIR_H
-#  include <ndir.h>
-# endif
+#  define dirent direct
+#  define NAMLEN(dirent) (dirent)->d_namlen
+#  if HAVE_SYS_NDIR_H
+#    include <sys/ndir.h>
+#  endif
+#  if HAVE_SYS_DIR_H
+#    include <sys/dir.h>
+#  endif
+#  if HAVE_NDIR_H
+#    include <ndir.h>
+#  endif
 #endif
 
 #include "sudo.h"
 #include <options.h>
 
-extern FILE *yyin, *yyout;
-
 /*
  * Globals
  */
 int parse_error = FALSE;
+extern FILE *yyin, *yyout;
+extern int printmatches;
 
 /*
- * this routine is called from the sudo.c module and tries to validate
+ * Prototypes
+ */
+static int has_meta	__P((char *));
+       void init_parser	__P((void));
+
+/*
+ * This routine is called from the sudo.c module and tries to validate
  * the user, host and command triplet.
  */
 int validate(check_cmnd)
@@ -102,26 +119,38 @@ int validate(check_cmnd)
     yyin = sudoers_fp;
     yyout = stdout;
 
+    /*
+     * Allocate space for data structures in the parser.
+     */
+    init_parser();
+
+    /*
+     * Need to be root while stat'ing things in the parser.
+     */
+    set_perms(PERM_ROOT);
     return_code = yyparse();
 
     /*
-     * don't need to keep this open...
+     * Don't need to keep this open...
      */
     (void) fclose(sudoers_fp);
 
     /* relinquish extra privs */
-    set_perms(PERM_ROOT);
     set_perms(PERM_USER);
 
     if (return_code || parse_error)
 	return(VALIDATE_ERROR);
 
-    if (top == 0)
-	/*
-	 * nothing on the top of the stack =>
-	 * user doesn't appear in sudoers
-	 */
-	return(VALIDATE_NO_USER);
+    /*
+     * Nothing on the top of the stack => user doesn't appear in sudoers.
+     * Allow anyone to try the psuedo commands "list" and "validate".
+     */
+    if (top == 0) {
+	if (check_cmnd == TRUE)
+	    return(VALIDATE_NO_USER);
+	else
+	    return(VALIDATE_NOT_OK);
+    }
 
     /*
      * Only check the actual command if the check_cmnd
@@ -133,18 +162,31 @@ int validate(check_cmnd)
 	while (top) {
 	    if (host_matches == TRUE)
 		/* user may always do validate or list on allowed hosts */
-		return(VALIDATE_OK);
+		if (no_passwd == TRUE)
+		    return(VALIDATE_OK_NOPASS);
+		else
+		    return(VALIDATE_OK);
 	    top--;
 	}
     else
 	while (top) {
-	    if (host_matches == TRUE)
-		if (cmnd_matches == TRUE)
-		    /* user was granted access to cmnd on host */
-		    return(VALIDATE_OK);
-		else if (cmnd_matches == FALSE)
-		    /* user was explicitly denied acces to cmnd on host */
+	    if (host_matches == TRUE) {
+		if (cmnd_matches == TRUE) {
+		   if (runas_matches == TRUE) {
+		    	/*
+			 * User was granted access to cmnd on host.
+		    	 * If no passwd required return as such.
+			 */
+		    	if (no_passwd == TRUE)
+			    return(VALIDATE_OK_NOPASS);
+		    	else
+			    return(VALIDATE_OK);
+		    }
+		} else if (cmnd_matches == FALSE) {
+		    /* User was explicitly denied acces to cmnd on host. */
 		    return(VALIDATE_NOT_OK);
+		}
+	    }
 	    top--;
 	}
 
@@ -160,10 +202,13 @@ int validate(check_cmnd)
 
 /*
  * If path doesn't end in /, return TRUE iff cmnd & path name the same inode;
- * otherwise, return TRUE if cmnd names one of the inodes in path
+ * otherwise, return TRUE if cmnd names one of the inodes in path.
  */
-int path_matches(cmnd, path)
-    char *cmnd, *path;
+int command_matches(cmnd, user_args, path, sudoers_args)
+    char *cmnd;
+    char *user_args;
+    char *path;
+    char *sudoers_args;
 {
     int plen;
     struct stat pst;
@@ -171,7 +216,6 @@ int path_matches(cmnd, path)
     struct dirent *dent;
     char buf[MAXPATHLEN+1];
     static char *c;
-    char *args;
 
     /* don't bother with pseudo commands like "validate" */
     if (*cmnd != '/')
@@ -187,93 +231,129 @@ int path_matches(cmnd, path)
 	    c++;
     }
 
-    /* if the given path has command line args, split them out */
-    if ((args = strchr(path, ' ')))
-	*args++ = '\0';
-
-    plen = strlen(path);
-    if (path[plen - 1] != '/') {
-#ifdef FAST_MATCH
-	char *p;
-
-	/* only proceed if the basenames of cmnd and path are the same */
-	if ((p = strrchr(path, '/')) == NULL)
-	    p = path;
-	else
-	    p++;
-	if (strcmp(c, p))
-	    return(FALSE);
-#endif /* FAST_MATCH */
-
-	if (stat(path, &pst) < 0)
-	    return(FALSE);
-
-	/* put things back the way we found 'em */
-	if (args)
-	    *(args - 1) = ' ';
-
+    /*
+     * If the pathname has meta characters in it use fnmatch(3)
+     * to do the matching
+     */
+    if (has_meta(path)) {
 	/*
-	 * Return true if inode/device matches and there are no args
+	 * Return true if fnmatch(3) succeeds and there are no args
 	 * (in sudoers or command) or if the args match;
 	 * else return false.
 	 */
-	if (cmnd_st.st_dev == pst.st_dev && cmnd_st.st_ino == pst.st_ino) {
-	    if (!args)
+	if (fnmatch(path, cmnd, FNM_PATHNAME))
+	    return(FALSE);
+	if (!sudoers_args)
+	    return(TRUE);
+	else if (!user_args && sudoers_args && !strcmp("\"\"", sudoers_args))
+	    return(TRUE);
+	else if (sudoers_args)
+	    return((fnmatch(sudoers_args, user_args ? user_args : "", 0) == 0));
+	else
+	    return(FALSE);
+    } else {
+	plen = strlen(path);
+	if (path[plen - 1] != '/') {
+#ifdef FAST_MATCH
+	    char *p;
+
+	    /* Only proceed if the basenames of cmnd and path are the same */
+	    if ((p = strrchr(path, '/')) == NULL)
+		p = path;
+	    else
+		p++;
+	    if (strcmp(c, p))
+		return(FALSE);
+#endif /* FAST_MATCH */
+
+	    if (stat(path, &pst) < 0)
+		return(FALSE);
+
+	    /*
+	     * Return true if inode/device matches and there are no args
+	     * (in sudoers or command) or if the args match;
+	     * else return false.
+	     */
+	    if (cmnd_st.st_dev != pst.st_dev || cmnd_st.st_ino != pst.st_ino)
+		return(FALSE);
+	    if (!sudoers_args)
 		return(TRUE);
-	    else if (cmnd_args && args)
-		return((strcmp(cmnd_args, args) == 0));
+	    else if (!user_args && sudoers_args && !strcmp("\"\"", sudoers_args))
+		return(TRUE);
+	    else if (sudoers_args)
+		return((fnmatch(sudoers_args, user_args ? user_args : "", 0) == 0));
 	    else
 		return(FALSE);
-	} else
+	}
+
+	/*
+	 * Grot through path's directory entries, looking for cmnd.
+	 */
+	dirp = opendir(path);
+	if (dirp == NULL)
 	    return(FALSE);
-    }
 
-    /*
-     * Grot through path's directory entries, looking for cmnd.
-     */
-    dirp = opendir(path);
-    if (dirp == NULL)
-	return(FALSE);
-
-    while ((dent = readdir(dirp)) != NULL) {
-	strcpy(buf, path);
-	strcat(buf, dent->d_name);
+	while ((dent = readdir(dirp)) != NULL) {
+	    strcpy(buf, path);
+	    strcat(buf, dent->d_name);
 #ifdef FAST_MATCH
-	/* only stat if basenames are not the same */
-	if (strcmp(c, dent->d_name))
-	    continue;
+	    /* only stat if basenames are not the same */
+	    if (strcmp(c, dent->d_name))
+		continue;
 #endif /* FAST_MATCH */
-	if (stat(buf, &pst) < 0)
-	    continue;
-	if (cmnd_st.st_dev == pst.st_dev && cmnd_st.st_ino == pst.st_ino)
-	    break;
-    }
+	    if (stat(buf, &pst) < 0)
+		continue;
+	    if (cmnd_st.st_dev == pst.st_dev && cmnd_st.st_ino == pst.st_ino)
+		break;
+	}
 
-    closedir(dirp);
-    return(dent != NULL);
+	closedir(dirp);
+	return(dent != NULL);
+    }
 }
 
 
 
+/*
+ * Returns TRUE if "n" is one of our ip addresses or if
+ * "n" is a network that we are on, else returns FALSE.
+ */
 int addr_matches(n)
     char *n;
 {
     int i;
-    struct in_addr addr;
+    char *m;
+    struct in_addr addr, mask;
 
-    addr.s_addr = inet_addr(n);
+    /* If there's an explicate netmask, use it. */
+    if ((m = strchr(n, '/'))) {
+	*m++ = '\0';
+	mask.s_addr = inet_addr(m);
+	addr.s_addr = inet_addr(n);
+	*(m - 1) = '/';
 
-    for (i = 0; i < num_interfaces; i++)
-	if (interfaces[i].addr.s_addr == addr.s_addr ||
-	    (interfaces[i].addr.s_addr & interfaces[i].netmask.s_addr)
-	    == addr.s_addr)
-	    return(TRUE);
+	for (i = 0; i < num_interfaces; i++)
+	    if ((interfaces[i].addr.s_addr & mask.s_addr) == addr.s_addr)
+		return(TRUE);
+    } else {
+	addr.s_addr = inet_addr(n);
+
+	for (i = 0; i < num_interfaces; i++)
+	    if (interfaces[i].addr.s_addr == addr.s_addr ||
+		(interfaces[i].addr.s_addr & interfaces[i].netmask.s_addr)
+		== addr.s_addr)
+		return(TRUE);
+    }
 
     return(FALSE);
 }
 
 
 
+/*
+ *  Returns TRUE if the given user belongs to the named group,
+ *  else returns FALSE.
+ */
 int usergr_matches(group, user)
     char *group;
     char *user;
@@ -291,7 +371,7 @@ int usergr_matches(group, user)
     /*
      * Check against user's real gid as well as group's user list
      */
-    if (getgid() == grpent->gr_gid)
+    if (grpent->gr_gid == user_gid)
 	return(TRUE);
 
     for (cur=grpent->gr_mem; *cur; cur++) {
@@ -304,6 +384,11 @@ int usergr_matches(group, user)
 
 
 
+/*
+ * Returns TRUE if "host" and "user" belong to the netgroup "netgr",
+ * else return FALSE.  Either of "host" or "user" may be NULL
+ * in which case that argument is not checked...
+ */
 int netgr_matches(netgr, host, user)
     char *netgr;
     char *host;
@@ -341,3 +426,21 @@ int netgr_matches(netgr, host, user)
     return(FALSE);
 #endif /* HAVE_INNETGR */
 }
+
+
+
+/*
+ * Returns TRUE if "s" has shell meta characters in it,
+ * else returns FALSE.
+ */
+static int has_meta(s)
+    char *s;
+{
+    register char *t;
+    
+    for (t = s; *t; t++) {
+	if (*t == '\\' || *t == '?' || *t == '*' || *t == '[' || *t == ']')
+	    return(TRUE);
+    }
+    return(FALSE);
+}
