@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.3.1
+ *  CU sudo version 1.5
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Please send bugs, changes, problems to sudo-bugs.cs.colorado.edu
+ *  Please send bugs, changes, problems to sudo-bugs@cs.colorado.edu
  *
  *******************************************************************
  *
@@ -25,7 +25,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: putenv.c,v 1.3 1994/08/08 17:05:22 millert Exp $";
+static char rcsid[] = "$Id: putenv.c,v 1.12 1996/08/17 23:25:40 millert Exp $";
 #endif /* lint */
 
 #include "config.h"
@@ -40,12 +40,14 @@ static char rcsid[] = "$Id: putenv.c,v 1.3 1994/08/08 17:05:22 millert Exp $";
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif /* HAVE_STRINGS_H */
-#ifdef HAVE_MALLOC_H
+#if defined(HAVE_MALLOC_H) && !defined(STDC_HEADERS)
 #include <malloc.h>
-#endif /* HAVE_MALLOC_H */
+#endif /* HAVE_MALLOC_H && !STDC_HEADERS */
 #include <errno.h>
 #include <sys/param.h>
 
+#include "compat.h"
+
 #if !defined(STDC_HEADERS) && !defined(__GNUC__)
 extern char *malloc	__P((size_t));
 #endif /* !STDC_HEADERS && !gcc */
