@@ -1,5 +1,5 @@
 /*
- * CU sudo version 1.4 (based on Root Group sudo version 1.1)
+ * CU sudo version 1.5 (based on Root Group sudo version 1.1)
  *
  * This software comes with no waranty whatsoever, use at your own risk.
  *
@@ -51,7 +51,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: sudo.c,v 1.137 1996/01/15 16:34:23 millert Exp $";
+static char rcsid[] = "$Id: sudo.c,v 1.167 1996/08/25 17:27:36 millert Exp $";
 #endif /* lint */
 
 #define MAIN
@@ -83,10 +83,14 @@ static char rcsid[] = "$Id: sudo.c,v 1.137 1996/01/15 16:34:23 millert Exp $";
 #include <sys/param.h>
 #include <netinet/in.h>
 #include <netdb.h>
-#if defined(__osf__) && defined(HAVE_C2_SECURITY)
-#include <sys/security.h>
-#include <prot.h>
-#endif /* __osf__ && HAVE_C2_SECURITY */
+#if (SHADOW_TYPE == SPW_SECUREWARE)
+#  ifdef __hpux
+#    include <hpsecurity.h>
+#  else
+#    include <sys/security.h>
+#  endif /* __hpux */
+#  include <prot.h>
+#endif /* SPW_SECUREWARE */
 #ifdef HAVE_DCE
 #include <pthread.h>
 #endif /* HAVE_DCE */
@@ -123,24 +127,30 @@ static void usage			__P((int));
 static void load_globals		__P((int));
 static int check_sudoers		__P((void));
 static void load_cmnd			__P((int));
-static void add_env			__P((void));
+static void add_env			__P((int));
 static void clean_env			__P((char **, struct env_table *));
-extern int user_is_exempt		__P((void));
+extern int  user_is_exempt		__P((void));
 extern struct passwd *sudo_getpwuid	__P((uid_t));
+extern void list_matches		__P((void));
 
 /*
  * Globals
  */
 int Argc;
 char **Argv;
+int NewArgc = 0;
+char **NewArgv = NULL;
 struct passwd *user_pw_ent;
+char *runas_user = "root";
 char *cmnd = NULL;
 char *cmnd_args = NULL;
 char *tty = NULL;
 char *prompt = PASSPROMPT;
 char host[MAXHOSTNAMELEN + 1];
+char *shost;
 char cwd[MAXPATHLEN + 1];
 struct stat cmnd_st;
+static char *runas_homedir = NULL;
 extern struct interface *interfaces;
 extern int num_interfaces;
 extern int printmatches;
@@ -161,6 +171,8 @@ struct env_table badenv_table[] = {
 #ifdef HAVE_KERB4
     { "KRB_CONF", 8 },
 #endif
+    { "ENV=", 4 },
+    { "BASH_ENV=", 9 },
     { (char *) NULL, 0 }
 };
 
@@ -180,9 +192,9 @@ int main(argc, argv)
     int sudo_mode = MODE_RUN;
     extern char ** environ;
 
-#if defined(SHADOW_TYPE) && (SHADOW_TYPE == SPW_SECUREWARE)
+#if (SHADOW_TYPE == SPW_SECUREWARE)
     (void) set_auth_parameters(argc, argv);
-#endif /* SHADOW_TYPE && SPW_SECUREWARE */
+#endif /* SPW_SECUREWARE */
 
     Argv = argv;
     Argc = argc;
@@ -196,30 +208,30 @@ int main(argc, argv)
      * parse our arguments
      */
     sudo_mode = parse_args();
-
-    switch(sudo_mode) {
-	case MODE_VERSION :
-	case MODE_HELP :
+ 
+    switch (sudo_mode) {
+	case MODE_VERSION:
+	case MODE_HELP:
 	    (void) printf("CU Sudo version %s\n", version);
 	    if (sudo_mode == MODE_VERSION)
 		exit(0);
 	    else
 		usage(0);
 	    break;
-	case MODE_VALIDATE :
+	case MODE_VALIDATE:
 	    cmnd = "validate";
 	    break;
-    	case MODE_KILL :
+    	case MODE_KILL:
 	    cmnd = "kill";
 	    break;
-	case MODE_LIST :
+	case MODE_LIST:
 	    cmnd = "list";
 	    printmatches = 1;
 	    break;
     }
 
     /* must have a command to run unless got -s */
-    if (cmnd == NULL && Argc == 1 && !(sudo_mode & MODE_SHELL))
+    if (cmnd == NULL && NewArgc == 0 && !(sudo_mode & MODE_SHELL))
 	usage(1);
 
     /*
@@ -238,6 +250,34 @@ int main(argc, argv)
 
     load_globals(sudo_mode);	/* load global variables used throughout sudo */
 
+    /*
+     * If we got the '-s' option (run shell) we need to redo NewArgv
+     * and NewArgc.  This can only be done after load_globals().
+     */
+    if ((sudo_mode & MODE_SHELL)) {
+	char **dst, **src = NewArgv;
+
+	NewArgv = (char **) malloc (sizeof(char *) * (++NewArgc + 1));
+	if (NewArgv == NULL) {
+	    perror("malloc");
+	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
+	    exit(1);
+	}
+
+	/* add the shell as argv[0] */
+	if (user_shell && *user_shell) {
+	    if ((NewArgv[0] = strrchr(user_shell, '/') + 1) == (char *) 1)
+		NewArgv[0] = user_shell;
+	} else {
+	    (void) fprintf(stderr, "%s: Unable to determine shell.", Argv[0]);
+	    exit(1);
+	}
+
+	/* copy the args from Argv */
+	for (dst = NewArgv + 1; (*dst = *src) != NULL; ++src, ++dst)
+	    ;
+    }
+
     rtn = check_sudoers();	/* check mode/owner on _PATH_SUDO_SUDOERS */
     if (rtn != ALL_SYSTEMS_GO) {
 	log_error(rtn);
@@ -248,8 +288,11 @@ int main(argc, argv)
 
 #ifdef SECURE_PATH
     /* replace the PATH envariable with a secure one */
-    if (!user_is_exempt())
-	sudo_setenv("PATH", SECURE_PATH);
+    if (!user_is_exempt() && sudo_setenv("PATH", SECURE_PATH)) {
+	perror("malloc");
+	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
+	exit(1);
+    }
 #endif /* SECURE_PATH */
 
     if ((sudo_mode & MODE_RUN)) {
@@ -257,25 +300,34 @@ int main(argc, argv)
     } else if (sudo_mode == MODE_KILL) {
 	remove_timestamp();	/* remove the timestamp ticket file */
 	exit(0);
-    } else if (sudo_mode == MODE_LIST) {
-	check_user();
-	log_error(ALL_SYSTEMS_GO);
-	(void) validate(FALSE);
-	exit(0);
     }
 
-    add_env();			/* add in SUDO_* envariables */
+    add_env(!(sudo_mode & MODE_SHELL));	/* add in SUDO_* envariables */
 
     /* validate the user but don't search for "validate" */
-    rtn = validate((sudo_mode != MODE_VALIDATE));
+    rtn = validate((sudo_mode != MODE_VALIDATE && sudo_mode != MODE_LIST));
+
     switch (rtn) {
 
 	case VALIDATE_OK:
-	    check_user();
+	case VALIDATE_OK_NOPASS:
+	    if (rtn != VALIDATE_OK_NOPASS) 
+		check_user();
 	    log_error(ALL_SYSTEMS_GO);
 	    if (sudo_mode == MODE_VALIDATE)
 		exit(0);
-	    set_perms(PERM_FULL_ROOT);
+	    else if (sudo_mode == MODE_LIST) {
+		list_matches();
+		exit(0);
+	    }
+
+	    /* become specified user or root */
+	    set_perms(PERM_RUNAS);
+
+	    /* set $HOME for `sudo -s' */
+	    if ((sudo_mode & MODE_SHELL) && runas_homedir)
+		(void) sudo_setenv("HOME", runas_homedir);
+
 #ifndef PROFILING
 	    if ((sudo_mode & MODE_BACKGROUND) && fork() > 0) {
 		exit(0);
@@ -302,47 +354,19 @@ int main(argc, argv)
 			exit(1);
 		    }
 		}
-
-		/*
-		 * If invoking a shell, replace "-s" with shell to exec.
-		 * For this to work we need to malloc() a new argv...
-		 */
-		if ((sudo_mode & MODE_SHELL)) {
-		    char **NewArgv;
-		    int i;
-
-		    NewArgv = (char **) malloc (sizeof(char *) * (Argc + 1));
-		    if (NewArgv == NULL) {
-			perror("malloc");
-			(void) fprintf(stderr, "%s: cannot allocate memory!\n",
-				       Argv[0]);
-			exit(1);
-		    }
-
-		    /* replace "-s" with the shell's name */
-		    if ((NewArgv[0] = strrchr(user_shell, '/') + 1)
-			== (char *) 1)
-			NewArgv[0] = user_shell;
-
-		    for (i = 1; i < Argc; i++)
-			NewArgv[i] = Argv[i];
-
-		    NewArgv[i] = (char *) NULL;
-
-		    EXEC(cmnd, NewArgv);
-		} else 
-		    EXEC(cmnd, &Argv[1]);
+		EXEC(cmnd, NewArgv);	/* run the command */
 	    }
 #else
 	    exit(0);
 #endif /* PROFILING */
-	    perror(cmnd);		/* exec failed! */
+	    /*
+	     * If we got here then the exec() failed...
+	     */
+	    (void) fprintf(stderr, "%s: ", Argv[0]);
+	    perror(cmnd);
 	    exit(-1);
 	    break;
 
-	case VALIDATE_NO_USER:
-	case VALIDATE_NOT_OK:
-	case VALIDATE_ERROR:
 	default:
 	    log_error(rtn);
 	    set_perms(PERM_FULL_USER);
@@ -366,7 +390,6 @@ static void load_globals(sudo_mode)
     int sudo_mode;
 {
     char *p;
-    struct passwd *pw_ent;
 #ifdef FQDN
     struct hostent *h_ent;
 #endif /* FQDN */
@@ -386,7 +409,7 @@ static void load_globals(sudo_mode)
 
 	/* fill in uid and name fields with the uid */
 	pw_ent.pw_uid = getuid();
-	(void) sprintf(pw_name, "%ld", pw_ent.pw_uid);
+	(void) sprintf(pw_name, "%ld", (long) pw_ent.pw_uid);
 	pw_ent.pw_name = pw_name;
 	user_pw_ent = &pw_ent;
 
@@ -441,29 +464,37 @@ static void load_globals(sudo_mode)
     }
 
     /*
-     * load the host global variable from gethostname()
-     * and use gethostbyname() if we want it to be fully qualified.
+     * load the host global variable from gethostname() and use
+     * gethostbyname() if we want to be sure it is fully qualified.
      */
     if ((gethostname(host, MAXHOSTNAMELEN))) {
 	strcpy(host, "localhost");
 	log_error(GLOBAL_NO_HOSTNAME);
 	inform_user(GLOBAL_NO_HOSTNAME);
+	exit(2);
+    }
 #ifdef FQDN
-    } else {
-	if ((h_ent = gethostbyname(host)) == NULL)
-	    log_error(GLOBAL_HOST_UNREGISTERED);
-	else
-	    strcpy(host, h_ent -> h_name);
-    }
-#else
-    }
+    if ((h_ent = gethostbyname(host)) == NULL)
+	log_error(GLOBAL_HOST_UNREGISTERED);
+    else
+	strcpy(host, h_ent -> h_name);
+#endif /* FQDN */
 
     /*
-     * We don't want to return the fully quallified name unless FQDN is set
+     * "host" is the (possibly fully-qualified) hostname and
+     * "shost" is the unqualified form of the hostname.
      */
-    if ((p = strchr(host, '.')))
+    if ((p = strchr(host, '.'))) {
 	*p = '\0';
-#endif /* FQDN */
+	if ((shost = strdup(host)) == NULL) {
+	    perror("malloc");
+	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
+	    exit(1);
+	}
+	*p = '.';
+    } else {
+	shost = &host[0];
+    }
 
     /*
      * load a list of ip addresses and netmasks into
@@ -485,7 +516,9 @@ static int parse_args()
 {
     int ret = MODE_RUN;			/* what mode is suod to be run in? */
     int excl = 0;			/* exclusive arg, no others allowed */
-    char *progname = Argv[0];		/* so we can save Argv[0] */
+
+    NewArgv = Argv + 1;
+    NewArgc = Argc - 1;
 
 #ifdef SHELL_IF_NO_ARGS
     if (Argc < 2) {			/* no options and no command */
@@ -497,26 +530,38 @@ static int parse_args()
 	usage(1);
 #endif /* SHELL_IF_NO_ARGS */
 
-    while (Argc > 1 && Argv[1][0] == '-') {
-	if (Argv[1][1] != '\0' && Argv[1][2] != '\0') {
+    while (NewArgc > 0 && NewArgv[0][0] == '-') {
+	if (NewArgv[0][1] != '\0' && NewArgv[0][2] != '\0') {
 	    (void) fprintf(stderr, "%s: Please use single character options\n",
-		progname);
+		Argv[0]);
 	    usage(1);
 	}
 
 	if (excl)
 	    usage(1);			/* only one -? option allowed */
 
-	switch (Argv[1][1]) {
+	switch (NewArgv[0][1]) {
 	    case 'p':
-		if (Argc + (ret & MODE_SHELL) < 4)
+		/* must have an associated prompt */
+		if (NewArgv[1] == NULL)
 		    usage(1);
 
-		prompt = Argv[2];
+		prompt = NewArgv[1];
 
 		/* shift Argv over and adjust Argc */
-		Argc--;
-		Argv++;
+		NewArgc--;
+		NewArgv++;
+		break;
+	    case 'u':
+		/* must have an associated runas user */
+		if (NewArgv[1] == NULL)
+		    usage(1);
+
+		runas_user = NewArgv[1];
+
+		/* shift Argv over and adjust Argc */
+		NewArgc--;
+		NewArgv++;
 		break;
 	    case 'b':
 		ret |= MODE_BACKGROUND;
@@ -545,9 +590,8 @@ static int parse_args()
 		ret |= MODE_SHELL;
 		break;
 	    case '-':
-		Argc--;
-		Argv++;
-		Argv[0] = progname;
+		NewArgc--;
+		NewArgv++;
 #ifdef SHELL_IF_NO_ARGS
 		if (ret == MODE_RUN)
 		    ret |= MODE_SHELL;
@@ -555,18 +599,21 @@ static int parse_args()
 		return(ret);
 	    case '\0':
 		(void) fprintf(stderr, "%s: '-' requires an argument\n",
-		    progname);
+		    Argv[0]);
 		usage(1);
 	    default:
-		(void) fprintf(stderr, "%s: Illegal option %s\n", progname,
-		    Argv[1]);
+		(void) fprintf(stderr, "%s: Illegal option %s\n", Argv[0],
+		    NewArgv[0]);
 		usage(1);
 	}
-	Argc--;
-	Argv++;
-	Argv[0] = progname;
+	NewArgc--;
+	NewArgv++;
     }
 
+    if (NewArgc > 0 && (ret == MODE_VALIDATE || ret == MODE_KILL ||
+			ret == MODE_LIST))
+	usage(1);
+
     return(ret);
 }
 
@@ -582,7 +629,7 @@ static int parse_args()
 static void usage(exit_val)
     int exit_val;
 {
-    (void) fprintf(stderr, "usage: %s -V | -h | -l | -v | -k | [-b] [-p prompt] -s | <command>\n", Argv[0]);
+    (void) fprintf(stderr, "usage: %s -V | -h | -l | -v | -k | [-b] [-p prompt] [-u username/#uid] -s | <command>\n", Argv[0]);
     exit(exit_val);
 }
 
@@ -595,16 +642,60 @@ static void usage(exit_val)
  *  this function adds sudo-specific variables into the environment
  */
 
-static void add_env()
+static void add_env(contiguous)
+    int contiguous;
 {
     char idstr[MAX_UID_T_LEN + 1];
+    size_t size;
+    char *buf;
 
-    /* add the SUDO_COMMAND envariable */
-    if (sudo_setenv("SUDO_COMMAND", cmnd)) {
+    /* add the SUDO_COMMAND envariable (cmnd + args) */
+    size = strlen(cmnd) + 1;
+    if (NewArgc > 1) {
+	char *to, **from;
+
+	if (contiguous) {
+	    size += (size_t) NewArgv[NewArgc-1] + strlen(NewArgv[NewArgc-1]) -
+		    (size_t) NewArgv[1] + 1;
+	} else {
+	    for (from = &NewArgv[1]; *from; from++)
+		size += strlen(*from) + 1;
+	}
+
+	if ((buf = (char *) malloc(size)) == NULL) {
+	    perror("malloc");
+	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
+	    exit(1);
+	}
+
+	/*
+	 * Copy the command and it's arguments info buf
+	 */
+	(void) strcpy(buf, cmnd);
+	to = buf + strlen(cmnd);
+	for (from = &NewArgv[1]; *from; from++) {
+	    *to++ = ' ';
+	    (void) strcpy(to, *from);
+	    to += strlen(*from);
+	}
+    } else {
+	buf = cmnd;
+    }
+    if (sudo_setenv("SUDO_COMMAND", buf)) {
 	perror("malloc");
 	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	exit(1);
     }
+    if (NewArgc > 1)
+	(void) free(buf);
+
+    /* grab a pointer to the flat arg string from the environment */
+    if (NewArgc > 1 && (cmnd_args = getenv("SUDO_COMMAND"))) {
+	if ((cmnd_args = strchr(cmnd_args, ' ')))
+	    cmnd_args++;
+	else
+	    cmnd_args = NULL;
+    }
 
     /* add the SUDO_USER envariable */
     if (sudo_setenv("SUDO_USER", user_name)) {
@@ -622,7 +713,7 @@ static void add_env()
     }
 
     /* add the SUDO_GID envariable */
-    (void) sprintf(idstr, "%ld", (long) getegid());
+    (void) sprintf(idstr, "%ld", (long) user_gid);
     if (sudo_setenv("SUDO_GID", idstr)) {
 	perror("malloc");
 	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
@@ -636,71 +727,25 @@ static void add_env()
  *
  *  load_cmnd()
  *
- *  This function sets the cmnd and cmnd_args global variables based on Argv
+ *  This function sets the cmnd global variable
  */
 
 static void load_cmnd(sudo_mode)
     int sudo_mode;
 {
-    int arg_start = 2;			/* position where command args start */
-    char *old_cmnd;			/* command before expansion */
-
-    /* If we are running a shell command args start at position 1 */
-    if ((sudo_mode & MODE_SHELL)) {
-	if (user_shell && *user_shell) {
-	    old_cmnd = user_shell;
-	    arg_start = 1;
-	} else {
-	    (void) fprintf(stderr, "%s: Unable to determine shell.", Argv[0]);
-	    exit(1);
-	}
-    } else {
-	old_cmnd = Argv[1];
-    }
-
-    if (strlen(old_cmnd) > MAXPATHLEN) {
+    if (strlen(NewArgv[0]) > MAXPATHLEN) {
 	errno = ENAMETOOLONG;
-	(void) fprintf(stderr, "%s: %s: Pathname too long\n", Argv[0], old_cmnd);
+	(void) fprintf(stderr, "%s: %s: Pathname too long\n", Argv[0],
+		       NewArgv[0]);
 	exit(1);
     }
 
-    /*
-     * Find the length of cmnd_args and allocate space, then fill it in.
-     */
-    if (Argc > arg_start) {
-	size_t arg_len;			/* sum length of all args */
-	char *from, *to;		/* loop variables for string copy */
-	int i;				/* venerable loop variable */
-
-	/*
-	 * How much space do we need?
-	 * This assumes that Argv (aka argv) is contiguous.
-	 */
-	arg_len = (size_t) Argv[Argc-1] + strlen(Argv[Argc-1]) + 1 -
-		  (size_t) Argv[arg_start];
-
-	if ((cmnd_args = (char *) malloc(arg_len)) == NULL) {
-	    perror("malloc");
-	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
-	    exit(1);
-	}
-
-	/*
-	 * Copy from Argv to cmnd_args, replacing NULL's with spaces.
-	 * This assumes that Argv (aka argv) is contiguous.
-	 */
-	for (i=0, to=cmnd_args, from=Argv[arg_start]; i < arg_len; i++) {
-	    if ((*to++ = *from++) == NULL)
-		*(to-1) = ' ';
-	}
-	cmnd_args[arg_len-1] = '\0';		/* need a NULL at the end */
-    }
-
     /*
      * Resolve the path
      */
-    if ((cmnd = find_path(old_cmnd)) == NULL) {
-	(void) fprintf(stderr, "%s: %s: command not found\n", Argv[0], old_cmnd);
+    if ((cmnd = find_path(NewArgv[0])) == NULL) {
+	(void) fprintf(stderr, "%s: %s: command not found\n", Argv[0],
+		       NewArgv[0]);
 	exit(1);
     }
 }
@@ -711,20 +756,45 @@ static void load_cmnd(sudo_mode)
  *
  *  check_sudoers()
  *
- *  This function check to see that the sudoers file is readable,
- *  owned by SUDOERS_OWNER, and not writable by anyone else.
+ *  This function check to see that the sudoers file is owned by
+ *  uid SUDOERS_UID, gid SUDOERS_GID and is mode SUDOERS_MODE.
  */
 
 static int check_sudoers()
 {
     struct stat statbuf;
-    struct passwd *pw_ent;
-    int fd;
+    int fd = -1;
     char c;
     int rtn = ALL_SYSTEMS_GO;
 
-    if (!(pw_ent = getpwnam(SUDOERS_OWNER)))
-	return(SUDOERS_NO_OWNER);
+    /*
+     * Fix the mode and group on sudoers file from old default.
+     * Only works if filesystem is readable/writable by root.
+     */
+    set_perms(PERM_ROOT);
+    if (!lstat(_PATH_SUDO_SUDOERS, &statbuf) && SUDOERS_UID == statbuf.st_uid) {
+	if (SUDOERS_MODE != 0400 && (statbuf.st_mode & 0007777) == 0400) {
+	    if (chmod(_PATH_SUDO_SUDOERS, SUDOERS_MODE) == 0) {
+		(void) fprintf(stderr, "%s: fixed mode on %s\n",
+		    Argv[0], _PATH_SUDO_SUDOERS);
+		if (statbuf.st_gid != SUDOERS_GID) {
+		    if (!chown(_PATH_SUDO_SUDOERS,GID_NO_CHANGE,SUDOERS_GID)) {
+			(void) fprintf(stderr, "%s: set group on %s\n",
+			    Argv[0], _PATH_SUDO_SUDOERS);
+			statbuf.st_gid = SUDOERS_GID;
+		    } else {
+			(void) fprintf(stderr,"%s: Unable to set group on %s: ",
+			    Argv[0], _PATH_SUDO_SUDOERS);
+			perror("");
+		    }
+		}
+	    } else {
+		(void) fprintf(stderr, "%s: Unable to fix mode on %s: ",
+		    Argv[0], _PATH_SUDO_SUDOERS);
+		perror("");
+	    }
+	}
+    }
 
     set_perms(PERM_SUDOERS);
 
@@ -734,12 +804,13 @@ static int check_sudoers()
 	rtn = NO_SUDOERS_FILE;
     else if (!S_ISREG(statbuf.st_mode))
 	rtn = SUDOERS_NOT_FILE;
-    else if (statbuf.st_uid != pw_ent -> pw_uid)
+    else if ((statbuf.st_mode & 0007777) != SUDOERS_MODE)
+	rtn = SUDOERS_WRONG_MODE;
+    else if (statbuf.st_uid != SUDOERS_UID || statbuf.st_gid != SUDOERS_GID)
 	rtn = SUDOERS_WRONG_OWNER;
-    else if ((statbuf.st_mode & 0000066))
-	rtn = SUDOERS_RW_OTHER;
 
-    (void) close(fd);
+    if (fd != -1)
+	(void) close(fd);
 
     set_perms(PERM_ROOT);
     set_perms(PERM_USER);
@@ -761,62 +832,108 @@ void set_perms(perm)
 {
     struct passwd *pw_ent;
 
-    switch(perm) {
-	case        PERM_ROOT :
+    switch (perm) {
+	case PERM_ROOT:
 				if (setuid(0)) {
 				    perror("setuid(0)");
 				    exit(1);
 				}
 			      	break;
 
-	case   PERM_FULL_ROOT :
-				if (setuid(0)) {  
-				    perror("setuid(0)");
-				    exit(1);
-				}
+	case PERM_USER: 
+    	    	    	        (void) setgid(user_gid);
 
-				if (!(pw_ent = getpwuid(0))) {
-				    perror("getpwuid(0)");
-				} else if (setgid(pw_ent->pw_gid)) {
-				    perror("setgid");
-    	    	    	    	}
-			      	break;
-
-	case        PERM_USER : 
     	    	    	        if (seteuid(user_uid)) {
     	    	    	            perror("seteuid(user_uid)");
     	    	    	            exit(1); 
     	    	    	        }
 			      	break;
 				
-	case   PERM_FULL_USER : 
+	case PERM_FULL_USER: 
 				if (setuid(0)) {
 				    perror("setuid(0)");
 				    exit(1);
 				}
 
+    	    	    	        (void) setgid(user_gid);
+
 				if (setuid(user_uid)) {
-				    perror("setuid(uid)");
+				    perror("setuid(user_uid)");
 				    exit(1);
 				}
 
 			      	break;
+	case PERM_RUNAS:
+				if (setuid(0)) {
+				    perror("setuid(0)");
+				    exit(1);
+				}
+				
+				/* XXX - add group/gid support */
+				if (*runas_user == '#') {
+				    if (setuid(atoi(runas_user + 1))) {
+					(void) fprintf(stderr,
+					    "%s: cannot set uid to %s: ",
+					    Argv[0], runas_user);
+					perror("");
+					exit(1);
+				    }
+				} else {
+				    if (!(pw_ent = getpwnam(runas_user))) {
+					(void) fprintf(stderr,
+					    "%s: no passwd entry for %s!\n",
+					    Argv[0], runas_user);
+					exit(1);
+				    }
 
-	case   PERM_SUDOERS : 
+				    if (setgid(pw_ent->pw_gid)) {
+					(void) fprintf(stderr,
+					    "%s: cannot set gid to %d: ",  
+					    Argv[0], pw_ent->pw_gid);
+					perror("");
+					exit(1);
+				    }
+
+				    if (setuid(pw_ent->pw_uid)) {
+					(void) fprintf(stderr,
+					    "%s: cannot set uid to %d: ",  
+					    Argv[0], pw_ent->pw_uid);
+					perror("");
+					exit(1);
+				    }
+
+				    runas_homedir = pw_ent->pw_dir;
+				}
+
+				break;
+	case PERM_SUDOERS: 
 				if (setuid(0)) {
 				    perror("setuid(0)");
 				    exit(1);
 				}
 
-				if (!(pw_ent = getpwnam(SUDOERS_OWNER))) {
-				    (void) fprintf(stderr, "%s: no passwd entry for sudoers file owner (%s)\n", Argv[0], SUDOERS_OWNER);
+				if (setgid(SUDOERS_GID)) {
+				    perror("setgid(SUDOERS_GID)");
 				    exit(1);
-				} else if (seteuid(pw_ent->pw_uid)) {
-				    (void) fprintf(stderr, "%s: ",
-							   SUDOERS_OWNER);
-				    perror("");
-				    exit(1);
-    	    	    	    	}
+				}
+
+				/*
+				 * If SUDOERS_UID == 0 we need to use
+				 * a different uid in order to avoid
+				 * NFS lossage.  Using uid 1 is a bit
+				 * bogus but should be safe.
+				 */
+				if (SUDOERS_UID == 0) {
+				    if (seteuid(1)) {
+					perror("seteuid(1)");
+					exit(1);
+				    }
+				} else {
+				    if (seteuid(SUDOERS_UID)) {
+					perror("seteuid(SUDOERS_UID)");
+					exit(1);
+				    }
+				}
 
 			      	break;
     }
