@@ -1,5 +1,5 @@
 /*
- * CU sudo version 1.4 (based on Root Group sudo version 1.1)
+ * CU sudo version 1.5 (based on Root Group sudo version 1.1)
  *
  * This software comes with no waranty whatsoever, use at your own risk.
  *
@@ -25,7 +25,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  $Id: sudo.h,v 1.97 1996/01/14 20:19:22 millert Exp $
+ *  $Id: sudo.h,v 1.108 1996/08/17 23:21:24 millert Exp $
  */
 
 #ifndef _SUDO_SUDO_H
@@ -48,18 +48,56 @@ struct interface {
  * apply when parsing is done & can be
  * accessed by *_matches macros
  */
+#define STACKINCREMENT (32)
 struct matchstack {
 	int user;
 	int cmnd;
 	int host;
+	int runas;
+	int nopass;
 };
 
-extern struct matchstack match[];
+/*
+ * Data structure describing a command in the
+ * sudoers file.
+ */
+struct sudo_command {
+    char *cmnd;
+    char *args;
+};
+
+
+extern struct matchstack *match;
 extern int top;
 
 #define user_matches	(match[top-1].user)
 #define cmnd_matches	(match[top-1].cmnd)
 #define host_matches	(match[top-1].host)
+#define runas_matches	(match[top-1].runas)
+#define no_passwd	(match[top-1].nopass)
+
+/*
+ * Structure containing command matches if "sudo -l" is used.
+ */
+struct command_match {
+    char *runas;
+    size_t runas_len;
+    size_t runas_size;
+    char *cmnd;
+    size_t cmnd_len;
+    size_t cmnd_size;
+    int nopasswd;
+};
+
+/*
+ * Structure containing Cmnd_Alias's if "sudo -l" is used.
+ */
+struct command_alias {
+    char *alias;
+    char *entries;
+    size_t entries_size;
+    size_t entries_len;
+};
 
 /*
  * Maximum number of characters to log per entry.  The syslogger
@@ -78,6 +116,7 @@ extern int top;
 #define VALIDATE_OK              0x00
 #define VALIDATE_NO_USER         0x01
 #define VALIDATE_NOT_OK          0x02
+#define VALIDATE_OK_NOPASS       0x03
 #define VALIDATE_ERROR          -1
 
 /*
@@ -94,10 +133,9 @@ extern int top;
 #define PASSWORDS_NOT_CORRECT    0x06
 #define NO_SUDOERS_FILE          ( 0x07 | GLOBAL_PROBLEM )
 #define BAD_SUDOERS_FILE         ( 0x08 | GLOBAL_PROBLEM )
-#define SUDOERS_NO_OWNER         ( 0x09 | GLOBAL_PROBLEM )
-#define SUDOERS_WRONG_OWNER      ( 0x0A | GLOBAL_PROBLEM )
+#define SUDOERS_WRONG_OWNER      ( 0x09 | GLOBAL_PROBLEM )
+#define SUDOERS_WRONG_MODE       ( 0x0A | GLOBAL_PROBLEM )
 #define SUDOERS_NOT_FILE         ( 0x0B | GLOBAL_PROBLEM )
-#define SUDOERS_RW_OTHER         ( 0x0C | GLOBAL_PROBLEM )
 #define SPOOF_ATTEMPT            0x0D
 #define BAD_STAMPDIR             0x0E
 #define BAD_STAMPFILE            0x0F
@@ -126,10 +164,10 @@ extern int top;
  * Used with set_perms()
  */
 #define PERM_ROOT                0x00
-#define PERM_FULL_ROOT           0x01
-#define PERM_USER                0x02
-#define PERM_FULL_USER           0x03
-#define PERM_SUDOERS             0x04
+#define PERM_USER                0x01
+#define PERM_FULL_USER           0x02
+#define PERM_SUDOERS             0x03
+#define PERM_RUNAS	         0x04
 
 /*
  * Shortcuts for user_pw_ent
@@ -142,11 +180,9 @@ extern int top;
 #define user_dir		(user_pw_ent -> pw_dir)
 
 /*
- * Prototypes
+ * Function prototypes
  */
-
-
-/* These are the functions that are called in sudo(8) */
+#define YY_DECL int yylex __P((void))
 
 #ifndef HAVE_STRDUP
 char *strdup		__P((const char *));
@@ -168,6 +204,8 @@ int validate		__P((int));
 void set_perms		__P((int));
 void remove_timestamp	__P((void));
 void load_interfaces	__P((void));
+int yyparse		__P((void));
+YY_DECL;
 
 
 /*
@@ -176,10 +214,12 @@ void load_interfaces	__P((void));
  */
 #ifndef MAIN
 extern char host[];
+extern char *shost;
 extern char cwd[];
 extern struct interface *interfaces;
 extern int num_interfaces;
 extern struct passwd *user_pw_ent;
+extern char *runas_user;
 extern char *tty;
 extern char *cmnd;
 extern char *cmnd_args;
@@ -187,6 +227,8 @@ extern char *prompt;
 extern struct stat cmnd_st;
 extern int Argc;
 extern char **Argv;
+extern int NewArgc;
+extern char **NewArgv;
 #endif
 extern int errno;
 
