@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.4
+ *  CU sudo version 1.5
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: tgetpass.c,v 1.33 1996/01/14 20:19:56 millert Exp $";
+static char rcsid[] = "$Id: tgetpass.c,v 1.44 1996/08/17 23:25:40 millert Exp $";
 #endif /* lint */
 
 #include "config.h"
@@ -42,6 +42,7 @@ static char rcsid[] = "$Id: tgetpass.c,v 1.33 1996/01/14 20:19:56 millert Exp $"
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif /* HAVE_STRINGS_H */
+#include <limits.h>
 #include <pwd.h>
 #include <sys/types.h>
 #ifdef HAVE_SYS_BSDTYPES_H
@@ -63,17 +64,21 @@ static char rcsid[] = "$Id: tgetpass.c,v 1.33 1996/01/14 20:19:56 millert Exp $"
 #include <sys/ioctl.h>
 #endif /* HAVE_TERMIO_H */
 #endif /* HAVE_TERMIOS_H */
+#if (SHADOW_TYPE == SPW_SECUREWARE)
+#  ifdef __hpux
+#    include <hpsecurity.h>
+#  else
+#    include <sys/security.h>
+#  endif /* __hpux */ 
+#  include <prot.h>
+#endif /* SPW_SECUREWARE */
 
 #include <pathnames.h>
 #include "compat.h"
 
-#ifndef _PASSWD_LEN
-#  ifdef HAVE_C2_SECURITY
-#    define	_PASSWD_LEN	24
-#  else
-#    define	_PASSWD_LEN	8
-#  endif /* HAVE_C2_SECURITY */
-#endif /* _PASSWD_LEN */
+#ifndef TCSASOFT
+#define TCSASOFT	0
+#endif /* TCSASOFT */
 
 
 /******************************************************************
@@ -91,14 +96,11 @@ char * tgetpass(prompt, timeout)
 {
 #ifdef HAVE_TERMIOS_H
     struct termios term;
-    tcflag_t svflagval;
 #else
 #ifdef HAVE_TERMIO_H
     struct termio term;
-    unsigned short svflagval;
 #else
     struct sgttyb ttyb;
-    int svflagval;
 #endif /* HAVE_TERMIO_H */
 #endif /* HAVE_TERMIOS_H */
 #ifdef POSIX_SIGNALS
@@ -107,29 +109,29 @@ char * tgetpass(prompt, timeout)
 #else
     int oldmask;
 #endif /* POSIX_SIGNALS */
-    FILE * input, * output;
+    int n, echo;
+    FILE *input, *output;
     static char buf[_PASSWD_LEN + 1];
     fd_set readfds;
     struct timeval tv;
-    char *tmp;
 
     /*
-     * mask out SIGINT, should probably just catch it.
+     * mask out SIGINT and SIGTSTP, should probably just catch and deal.
      */
 #ifdef POSIX_SIGNALS
-    (void) memset((VOID *)&mask, 0, sizeof(mask));
+    (void) sigemptyset(&mask);
     (void) sigaddset(&mask, SIGINT);
+    (void) sigaddset(&mask, SIGTSTP);
     (void) sigprocmask(SIG_BLOCK, &mask, &oldmask);
 #else
-    oldmask = sigblock(sigmask(SIGINT));
+    oldmask = sigblock(sigmask(SIGINT)|sigmask(SIGTSTP));
 #endif
 
     /*
      * open /dev/tty for reading/writing if possible or use
      * stdin and stderr instead.
      */
-    input = fopen(_PATH_TTY, "r+");
-    if (!input) {
+    if ((input = fopen(_PATH_TTY, "r+")) == NULL) {
 	input = stdin;
 	output = stderr;
 	(void) fflush(output);
@@ -142,20 +144,23 @@ char * tgetpass(prompt, timeout)
      */
 #ifdef HAVE_TERMIOS_H
     (void) tcgetattr(fileno(input), &term);
-    svflagval = term.c_lflag;
-    term.c_lflag &= ~ECHO;
-    (void) tcsetattr(fileno(input), TCSAFLUSH, &term);
+    if ((echo = (term.c_lflag & ECHO))) {
+	term.c_lflag &= ~ECHO;
+	(void) tcsetattr(fileno(input), TCSAFLUSH|TCSASOFT, &term);
+    }
 #else
 #ifdef HAVE_TERMIO_H
     (void) ioctl(fileno(input), TCGETA, &term);
-    svflagval = term.c_lflag;
-    term.c_lflag &= ~ECHO;
-    (void) ioctl(fileno(input), TCSETA, &term);
+    if ((echo = (term.c_lflag & ECHO))) {
+	term.c_lflag &= ~ECHO;
+	(void) ioctl(fileno(input), TCSETA, &term);
+    }
 #else
     (void) ioctl(fileno(input), TIOCGETP, &ttyb);
-    svflagval = ttyb.sg_flags;
-    ttyb.sg_flags &= ~ECHO;
-    (void) ioctl(fileno(input), TIOCSETP, &ttyb);
+    if ((echo = (ttyb.sg_flags & ECHO))) {
+	ttyb.sg_flags &= ~ECHO;
+	(void) ioctl(fileno(input), TIOCSETP, &ttyb);
+    }
 #endif /* HAVE_TERMIO_H */
 #endif /* HAVE_TERMIOS_H */
 
@@ -168,46 +173,60 @@ char * tgetpass(prompt, timeout)
 	(void) rewind(output);
     }
 
-    /* setup for select(2) */
-    FD_ZERO(&readfds);
-    FD_SET(fileno(input), &readfds);
+    /*
+     * Timeout of <= 0 means no timeout
+     */
+    if (timeout > 0) {
+	/* setup for select(2) */
+	FD_ZERO(&readfds);
+	FD_SET(fileno(input), &readfds);
 
-    /* set timeout for select */
-    tv.tv_sec = timeout;
-    tv.tv_usec = 0;
+	/* set timeout for select */
+	tv.tv_sec = timeout;
+	tv.tv_usec = 0;
 
-    /* return NULL if nothing to read by timeout */
+	/* how many file descriptors may we have? */
 #ifdef HAVE_SYSCONF
-    if (select(sysconf(_SC_OPEN_MAX), &readfds, NULL, NULL, &tv) <= 0) {
+	n = sysconf(_SC_OPEN_MAX);
 #else
-    if (select(getdtablesize(), &readfds, NULL, NULL, &tv) <= 0) {
+	n = getdtablesize();
 #endif /* HAVE_SYSCONF */
+
+	/*
+	 * get password or return empty string if nothing to read by timeout
+	 */
 	buf[0] = '\0';
-	goto cleanup;			/* XXX - goto considered harmful */
-    }
-
-    /* get the password */
-    if (!fgets(buf, sizeof(buf), input)) {
+	if (select(n, &readfds, 0, 0, &tv) > 0 && fgets(buf, sizeof(buf), input)) {
+	    n = strlen(buf);
+	    if (buf[n - 1] == '\n')
+		buf[n - 1] = '\0';
+	}
+    } else {
 	buf[0] = '\0';
-	goto cleanup;			/* XXX - goto considered harmful */
+	if (fgets(buf, sizeof(buf), input)) {
+	    n = strlen(buf);
+	    if (buf[n - 1] == '\n')
+		buf[n - 1] = '\0';
+	}
     }
 
-    if (*(tmp = &buf[strlen(buf)-1]) == '\n')
-	*tmp = '\0';
-
-cleanup:
-
      /* turn on echo */
 #ifdef HAVE_TERMIOS_H
-    term.c_lflag = svflagval;
-    (void) tcsetattr(fileno(input), TCSAFLUSH, &term);
+    if (echo) {
+	term.c_lflag |= ECHO;
+	(void) tcsetattr(fileno(input), TCSAFLUSH|TCSASOFT, &term);
+    }
 #else
 #ifdef HAVE_TERMIO_H
-    term.c_lflag = svflagval;
-    (void) ioctl(fileno(input), TCSETA, &term);
+    if (echo) {
+	term.c_lflag |= ECHO;
+	(void) ioctl(fileno(input), TCSETA, &term);
+    }
 #else
-    ttyb.sg_flags = svflagval;
-    (void) ioctl(fileno(input), TIOCSETP, &ttyb);
+    if (echo) {
+	ttyb.sg_flags |= ECHO;
+	(void) ioctl(fileno(input), TIOCSETP, &ttyb);
+    }
 #endif /* HAVE_TERMIO_H */
 #endif /* HAVE_TERMIOS_H */
 
@@ -216,6 +235,8 @@ cleanup:
 	(void) fflush(output);
 	(void) rewind(output);
     }
+
+    /* print a newline since echo is turned off */
     (void) fputc('\n', output);
 
     /* restore old signal mask */
