@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.4
+ *  CU sudo version 1.5
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: visudo.c,v 1.60 1996/01/14 20:19:56 millert Exp $";
+static char rcsid[] = "$Id: visudo.c,v 1.73 1996/08/22 18:00:46 millert Exp $";
 #endif /* lint */
 
 #include "config.h"
@@ -82,10 +82,11 @@ static char whatnow		__P((void));
 static void whatnow_help	__P((void));
 static RETSIGTYPE Exit		__P((int));
 static void setup_signals	__P((void));
-int path_matches		__P((char *, char *));
+int command_matches		__P((char *, char *, char *, char *));
 int addr_matches		__P((char *));
 int netgr_matches		__P((char *, char *, char *));
 int usergr_matches		__P((char *, char *));
+void init_parser		__P((void));
 
 
 /*
@@ -99,16 +100,20 @@ extern int errorlineno, sudolineno;
  * Globals
  */
 char **Argv;
+char **NewArgv = NULL;
+int NewArgc = 0;
 char *sudoers = _PATH_SUDO_SUDOERS;
 char *stmp = _PATH_SUDO_STMP;
 int parse_error = FALSE;
-
+char *runas_user = "root";
 
 /*
  * For the parsing routines
  */
 char host[] = "";
+char *shost = "";
 char *cmnd = "";
+char *cmnd_args = NULL;
 struct passwd *user_pw_ent;
 
 
@@ -128,7 +133,6 @@ int main(argc, argv)
     int sudoers_fd;			/* sudoers file descriptor */
     int stmp_fd;			/* stmp file descriptor */
     int n;				/* length parameter */
-    struct passwd *pwd;			/* to look up info for SUDOERS_OWNER */
 
     (void) setbuf(stderr, (char *)NULL);	/* unbuffered stderr */
 
@@ -151,6 +155,9 @@ int main(argc, argv)
     else if (argc != 1)
 	usage();
 
+    /* user_pw_ent needs to point to something... */
+    user_pw_ent = getpwuid(getuid());
+
 #ifdef ENV_EDITOR
     /*
      * If we are allowing EDITOR and VISUAL envariables set Editor
@@ -161,19 +168,6 @@ int main(argc, argv)
 	    Editor = EDITOR;
 #endif /* ENV_EDITOR */
 
-    /*
-     * Need to find who should own the sudoers file
-     */
-    if (!(pwd = getpwnam(SUDOERS_OWNER))) {
-	(void) fprintf(stderr,
-		       "%s:  no passwd entry for sudoers file owner (%s)\n",
-		       Argv[0], SUDOERS_OWNER);
-	exit(1);
-    }
-
-    /* user_pw_ent needs to be defined to _something_, this will do */
-    user_pw_ent = pwd;
-
     /*
      * Copy sudoers file to stmp
      */
@@ -212,16 +206,6 @@ int main(argc, argv)
     (void) close(sudoers_fd);
     (void) close(stmp_fd);
 
-    /*
-     * Change ownership of temp file to SUDOERS_OWNER
-     * so when we move it to sudoers things are kosher.
-     */
-    if (chown(stmp, pwd -> pw_uid, -1)) {
-	(void) fprintf(stderr, "%s: Warning, unable to set owner to %s: ",
-	    Argv[0], SUDOERS_OWNER);
-	perror("");
-    }
-
     /*
      * Edit the temp file and parse it (for sanity checking)
      */
@@ -234,8 +218,9 @@ int main(argc, argv)
 	else
 	    (void) sprintf(buf, "%s %s", Editor, stmp);
 
-	/* do the edit */
-	if (system(buf) == 0) {
+	/* do the edit -- some SYSV editors return 256 instead of 0 */
+	n = system(buf);
+	if (n == 0 || n == 256) {
 	    struct stat statbuf;	/* for sanity checking */
 
 	    /* make sure stmp exists */
@@ -271,10 +256,7 @@ int main(argc, argv)
 	    }
 
 	    /* clean slate for each parse */
-	    parse_error = FALSE;
-	    errorlineno = -1;
-	    sudolineno = 1;
-	    top = 0;
+	    init_parser();
 
 	    /* parse the sudoers file */
 	    if (yyparse()) {
@@ -302,6 +284,25 @@ int main(argc, argv)
 	}
     } while (parse_error == TRUE);
 
+    /*
+     * Change mode and ownership of temp file so when
+     * we move it to sudoers things are kosher.
+     */
+    if (chown(stmp, SUDOERS_UID, SUDOERS_GID)) {
+	(void) fprintf(stderr,
+	    "%s: Unable to set (uid, gid) of %s to (%d, %d): ",
+	    Argv[0], stmp, SUDOERS_UID, SUDOERS_GID);
+	perror("");
+	Exit(1);
+    }
+    if (chmod(stmp, SUDOERS_MODE)) {
+	(void) fprintf(stderr,
+	    "%s: Unable to change mode of %s to %o: ",
+	    Argv[0], stmp, SUDOERS_MODE);
+	perror("");
+	Exit(1);
+    }
+
     /*
      * Now that we have a sane stmp file (parse ok) it needs to be
      * rename(2)'d to sudoers.  If the rename(2) fails we try using
@@ -342,16 +343,6 @@ int main(argc, argv)
 	    Exit(1);
 	}
 
-    /*
-     * Make the new sudoers file readable only by owner.
-     * If this fail it is ok since the file is only least rw owner.
-     */
-    if (chmod(sudoers, 0400)) {
-	(void) fprintf(stderr, "%s: Warning, unable to chmod 0400 %s: ",
-	    Argv[0], sudoers);
-	perror("");
-    }
-
     return(0);
 }
 
@@ -363,8 +354,11 @@ int main(argc, argv)
  *  These exist to allow us to use the same parser as sudo(8).
  */
 
-int path_matches(cmnd, path)
-    char *cmnd, *path;
+int command_matches(cmnd, user_args, path, sudoers_args)
+    char *cmnd;
+    char *user_args;
+    char *path;
+    char *sudoers_args;
 {
     return(TRUE);
 }
